const path = require("path");
const fs = require('fs');

let subPackages = null;
function getSub() {
  return subPackages;
}

function setSub(sub) {
  subPackages = sub;
}

exports.initSub = function(root) {
  // null表明是首次初始化；undefined表明此工程不是分包工程
  if(subPackages === undefined || (subPackages && subPackages.length > 0)) {
    return;
  }
  let subPath = path.resolve(root, 'app.json');
  let appJson = JSON.parse(fs.readFileSync(subPath));
  let temSubPackages = appJson.subPackages || appJson.subpackages || [];
  setSub(temSubPackages);
}

function getSubConfig(dep, subPackages, originalPath) {
  let projectPath = dep.replace(originalPath, "");
  projectPath = path.posix.normalize(projectPath);
  let subConfig;
  if (subPackages) {
    for (let e = 0, f = subPackages.length; e < f; e++) {
      let f = subPackages[e],
        g = path.posix.normalize(f.root + '/');
      if (0 === projectPath.indexOf(g) || 1 === projectPath.indexOf(g)) {
        subConfig = f;
        break
      }
    }
  }
  return subConfig
}

function getType(filePath, root) {
  let subConfig = getSub();
  let oneConfig = getSubConfig(filePath, subConfig, root);
  return oneConfig
}

exports.checkIsInSub = function(sourcePath, importedPath, root) {
  // subPackages false表明没有分包，或者没有初始化
  if(!subPackages) {
    return true
  }
  // 获取入口文件是在分包，还是主包
  let sourceType = getType(sourcePath, root);
  // 获取分包文件是在分包，还是主包
  let importedType =  getType(importedPath, root);
  // 表明入口文件是分包，分包可以引用主包以及自己的分包。不能引用其他分包
  if(sourceType) {
    // 被引用资源，存在于其他分包中
    if(importedType && sourceType.root !== importedType.root) {
      return false;
    }
    return true;
  } else {
    // 表明入口文件在主包, 切被引用资源在分包
    if(importedType) {
      return false;
    }
    return true;
  }
}